# coding: utf-8
# Project：xbsTestTools
# File：test_tools.py
# Author：张炼武
# Date ：2025/6/9 11:04
# IDE：PyCharm
import subprocess
from time import sleep

from xbsTestTools.system_tools.buring.download_tool import download_file, get_link, get_link_file
from xbsTestTools.system_tools.buring.zr import zhanRui_flash_run


class TestTools:
    def __init__(self, dev):
        from xbsTestTools import XbsUiOperate
        from xbsTestTools import AdbTools

        self.ats = AdbTools(dev)
        self.xuo = XbsUiOperate(dev)

        self.dev = dev
        self.test1_ = ""
        self.test2_ = "com.emdoor.pressure.tester"

    def start_to_name(self, test_name, test_number: int = 0):
        """启动指定测试"""
        print(f"启动指定测试 {test_name} {test_number}")
        run2_type = self.xuo.app_start(self.test2_)
        if not run2_type:
            print("平板压力测试工具2 打开失败")
            return True

        self.xuo.click_roll(text=test_name)

        self.xuo.click(text="复位")

        if test_number != 0:
            self.xuo.click(text="设置次数")
            self.xuo.click(text=f"{test_number}")
            self.xuo.click(text="确定")

        self.xuo.click(text="开始")
        return True

    def start_onoff(self):
        """启动开关机"""
        print("启动开关机")
        run2_type = self.xuo.app_start(self.test2_)
        if not run2_type:
            print("平板压力测试工具2 打开失败")
            return True

        self.xuo.click_roll(text="开关机测试")

        self.xuo.click(text="复位")

        self.xuo.click(text="设置次数")
        self.xuo.click(text="1")
        self.xuo.click(text="确定")
        self.xuo.click(text="开始")

        sleep(80)
        if not self.ats.wait_dev():
            # 超时3分钟
            print("开关机失败，adb设备不在线")
            return False
        return True

    def start_restore(self):
        """启动恢复出厂设置"""
        print("启动恢复出厂设置")
        run2_type = self.xuo.app_start(self.test2_)
        if not run2_type:
            print("平板压力测试工具2 打开失败")
            return True

        self.xuo.click_roll(text="恢复出厂设置")

        self.xuo.click(text="复位")

        self.xuo.click(text="设置次数")

        self.xuo.click(text="1")

        self.xuo.click(text="确定")
        self.xuo.click(text="开始")

        sleep(10)

        if not self.ats.wait_dev(timeout=300):
            # 超时3分钟
            print("恢复出厂设置失败，adb设备不在线")
            return False
        return True

    def start_hibernate(self, time_: int = 5):
        """休眠唤醒"""
        print(f"休眠{time_}s")
        try:
            subprocess.run(f"adb -s {self.dev} shell input keyevent 26")
            sleep(time_)
            subprocess.run(f"adb -s {self.dev} shell input keyevent 266")
            sleep(1)
            return True
        except:
            return False

    def start_restart(self):
        """重启"""
        print("重启")
        subprocess.run(f"adb -s {self.dev} reboot")
        sleep(10)
        if not self.ats.wait_dev():
            print("执行重启后设备不在线")
            return False
        return True

    def start_brun(self, burn_file=""):
        """启动刷机"""
        if not burn_file:
            links = get_link()
            for k,v in links.items():
                print(f"【{k}】 {v}")

            file_index = str(input("请选择刷机哪个版本: \n"))
            burn_file = get_link_file(links[file_index])

        # 下载软件
        download_type = download_file(burn_file)
        if not download_type:
            print(f"{self.dev} 烧录文件下载失败，无法烧录")
            return False
        # 开始烧录
        run_type = zhanRui_flash_run(self.dev, download_type)
        if not run_type:
            download_type(f"{self.dev} 烧录失败")
        return True


if __name__ == "__main__":
    dev = "00test4H01881655D00189"
    tt = TestTools(dev)
    tt.start_onoff()




# coding: utf-8
# Project：xbsSysStabilityAT
# File：qz.py
# Author：张炼武
# Date ：2025/4/27 11:48
# IDE：PyCharm
import os
import queue
import subprocess
import threading
from time import sleep
from xbsTestTools.system_tools.buring.download_tool import get_download_file


def allwinner_flash(dev, softwarePath, burn_type):
    mirror_file = get_download_file("allwinner", softwarePath)
    scatterFile = os.path.join(softwarePath, mirror_file)
    root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + os.sep
    tool_path = root_path + r"utils\PhoenixConsole_V2.0.8\PhoenixConsole.exe"
    download_cmd = f"{tool_path} -e 11 -r -s {dev} -o 1 -p {scatterFile}"
    print(f"{dev} 执行烧录  烧录命令：{download_cmd}")
    download_info = subprocess.run(download_cmd, stdout=subprocess.PIPE)
    if "result=Success" in str(download_info.stdout):  # 全志下载信息
        print(f"{dev} download log info result=Success")
        burn_type.put(True)
        return True
    else:
        print(f"{dev} 烧写失败，没找到烧录成功关键词")
        burn_type.put(False)
        return False


def all_burn_run(dev, softwarePath):
    _type = queue.Queue()
    # 烧录启动
    t = threading.Thread(target=allwinner_flash, args=(dev, softwarePath, _type))
    t.start()
    sleep(5)
    t.join(600)
    print(f"{dev} 烧录完成")
    return _type.get()



# coding: utf-8
# Project：xbsSysStabilityAT
# File：zr.py
# Author：张炼武
# Date ：2025/4/27 11:48
# IDE：PyCharm
import os
import queue
import subprocess
import threading
from time import sleep
import serial.tools.list_ports
from xbsTestTools.system_tools.buring.download_tool import get_download_file


def zhanRui_flash(dev, com, softwarePath, burn_type):
    mirror_file = get_download_file("UNISOC", softwarePath)
    softwarePath = os.path.join(softwarePath, mirror_file)  # 拼接烧录镜像路径
    flash_cmd = 'CmdDloader.exe -pac {} -reset -port {}'.format(softwarePath, com)
    try:
        print(f"{dev} 执行展瑞烧录命令: {flash_cmd}")
        flash_result = subprocess.run(flash_cmd, shell=True, stdout=subprocess.PIPE, encoding="utf-8",
                                      timeout=400).stdout

        if 'Download Result = Passed' in flash_result:
            print(f"{dev} 刷机成功")
            burn_type.put(True)
            return True
        else:
            print(f"{dev} 刷机失败，刷机完成中没有 成功字段")
            burn_type.put(False)
            return
    except Exception as e:
        print(f"{dev} 烧写版本超时 ERROR_INFO: {e}")
        burn_type.put(False)
        return


def zhanRui_flash_run(dev, softwarePath):
    burn_type = queue.Queue()  # 定义状态变量
    patList = os.environ.get('Path')  # 获取当前系统环境变量list
    root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + os.sep
    CmdDloader = root_path + r"utils\ZhanRui_Flash_Tool\Download_R27.22.4204\Download_R27.22.4204\Bin"  # 定义展锐烧录器path
    if CmdDloader not in patList:
        os.environ['Path'] = os.pathsep.join([CmdDloader, patList])
    # 获取设备串口号
    read = os.popen(f"adb -s {dev} shell getprop persist.sys.seewo.is_adb_on").read()  # 检查设备默认adb是否开启
    if "true" not in read:
        # 默认开启设备adb
        print("检查给设备设置默认开启adb")
        os.system(f"adb -s {dev} shell am broadcast -a com.seewo.action.SWITCH_ADB --ez open_usb true com.cvte.logtool")
    sleep(1)
    print(f"{dev} 设备进入烧录模式 adb reboot autodloader")
    subprocess.run(f"adb -s {dev} reboot autodloader")  # 设备进入烧录模式
    print(f"{dev} 正在获取设备串口COM")
    sleep(15)
    ports = []
    for port in serial.tools.list_ports.comports():
        if "SPRD U2S" in port.description:
            ports.append(port.name)

    if len(ports) < 1:
        return False

    t = threading.Thread(target=zhanRui_flash, args=(dev, ports[0], softwarePath, burn_type))  # 创建刷机线程
    t.start()  # 启动刷机线程
    print(f"{dev} {ports[0]} 正在烧录中...")
    t.join()

    print(f"{dev} 正在重启")
    sleep(10)
    return burn_type.get()



# coding: utf-8
# Project：xbsSysStabilityAT
# File：download_tool.py
# Author：张炼武
# Date ：2025/4/27 12:01
# IDE：PyCharm
import os
import shutil
import subprocess
from time import sleep
import patoolib
import requests
import time
import datetime

root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + os.sep


def get_link():
    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36',
        'host': 'setup.gz.cvte.cn',
    }
    url = "https://setup.gz.cvte.cn/CrOS_new/xbs_os2/Seewo_XPG13/Image/XPG13/release/"  # XPG13
    response = requests.get(url, headers=headers)
    text_list = str(response.text).split("\n")

    link_list = {}
    index = 0
    for text in text_list:
        if 'href="seewoOS' in text and "V9.9.9" not in text:
            index += 1
            text = text[text.find('<a href="') + 9:text.find('/">')]
            link_list.update({str(index): url + "/" + text})
    return link_list


def get_link_file(link):
    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/135.0.0.0 Safari/537.36',
        'host': 'setup.gz.cvte.cn',
    }
    url = link
    response = requests.get(url, headers=headers)
    text_list = str(response.text).split("\n")

    file_link = ""
    for text in text_list:
        if 'href="seewoOS' in text:
            text = text[text.find('<a href="') + 9:text.find('">')]
            file_link = text
            break
    return url + "/" + file_link


def get_file_size(url):
    response = requests.head(url)
    file_size = int(response.headers.get('content-length', 0))
    return file_size


def get_directories(path):
    """
    获取指定目录下的文件夹
    """
    return [entry.name for entry in os.scandir(path) if entry.is_dir()]


def query_update_folder(folder_path):
    """
    获取路径下最后修改时间的文件夹
    return 文件夹
    """
    # 初始化最后修改时间和对应的文件名
    last_modified_time = 0
    last_modified_file = None

    # 遍历文件夹中的文件
    for filename in os.listdir(folder_path):
        file_path = os.path.join(folder_path, filename)
        # 确定为文件夹
        if os.path.isdir(file_path):
            mod_time = os.path.getmtime(file_path)

            # 比较时间戳，更新最后修改的文件信息
            if mod_time > last_modified_time:
                last_modified_time = mod_time
                last_modified_file = filename
    return last_modified_file


def getMtkLog_keywords(path):
    """
    获取mtk烧录完成后日志中打印刷机完成的信息
    """
    # SP_FT_Dump_03-21-2024-17-20-39
    with open(r'C:\ProgramData\SP_FT_Logs\%s\QT_FLASH_TOOL.log' % path) as f:
        # 使用生成器表达式和next()来找出第一个包含"a"的元素
        read = next((s for s in f.readlines()[-15:] if "FlashTool_EnableWatchDogTimeout Succeeded" in s), None)
        if read is None:
            return
        return True


def cmd_to_file(cmd, file, timeout):
    """
    执行cmd命令，重定向内容到指定文件
    cmd：执行的cmd
    file: 文件
    return: 目标文件
    """
    with open(root_path + "data\\" + file, 'w', encoding='utf-8') as fe:
        process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        try:
            output, unused_err = process.communicate(timeout=timeout)
            fe.write(output.decode())
        except Exception as e:
            fe.write(str(e))
            print(e)
            return False, str(e)
    return True, root_path + "data\\" + file


def get_download_file(platform, file_path):
    """platform :平台类型
       file_path：版本路径
       返回烧录镜像文件"""
    filenames = os.listdir(file_path)
    for filename in filenames:
        if platform == "MTK":
            if "Android_scatter.txt" in filename:
                return filename
        elif platform == "UNISOC":
            if ".pac" in filename:
                return filename
        elif platform == "allwinner":
            if "img" in filename:
                return filename
    print("烧录镜像文件选择异常")


def delete_files_older_than_3days(folder_path):
    # 计算3天前的时间戳
    try:
        current_time = datetime.datetime.now()
        offset = datetime.timedelta(days=-3)
        three_days_ago = (current_time + offset)
        three_days_ago_timestamp = time.mktime(three_days_ago.timetuple())

        # 遍历文件夹
        for root, dirs, files in os.walk(folder_path):
            for file_name in files:
                file_path = os.path.join(root, file_name)

                # 获取文件最后修改时间
                file_mtime = os.path.getmtime(file_path)

                # 检查时间并删除
                if file_mtime < three_days_ago_timestamp:
                    try:
                        os.remove(file_path)
                        print(f"已删除: {file_path} (修改时间: {time.ctime(file_mtime)})")
                    except Exception as e:
                        print(f"删除失败 {file_path}: {str(e)}")
    except:
        ...


def download_file(link, jointly_model=None, sale_model=None):
    """
    下载软件方法
    downloadLink 软件链接
    """
    patList = os.environ.get('Path')
    wgetPath = root_path + "utils\\wget"
    if wgetPath not in patList:
        print("正在将wget加入系统环境变量中...")
        os.environ['Path'] = os.pathsep.join([wgetPath, patList])

    fileName = root_path + "data\\" + link.split('/')[-1]  # 软件文件名称
    delete_files_older_than_3days(f"{root_path}data")  # 删除3天前的旧文件
    # 软件文件名称去除.7z
    if ".7z" in fileName:
        softwarePath = fileName.split('.7z')[0]
    elif ".zip" in fileName:
        softwarePath = fileName.split('.zip')[0]
    else:
        softwarePath = fileName

    # 判断刷机文件是否下载
    if not os.path.exists(fileName):
        if ".7z" not in fileName and ".zip" not in fileName:
            print("请检查当前选择的链接是否正常，需要一个可用文件目标地址！！")
            return False, "下载软件失败"

        download = 'wget {} -P {} --no-check-certificate'.format(link, root_path + "data")

        print("软件未下载，开始下载...")
        try:
            subprocess.run(download, shell=True)
        except:
            print("下载失败, 删除残留文件重新下载...")
            os.remove(fileName)

            # 重试重新下载
            try:
                subprocess.run(download, shell=True)
            except:
                print("下载失败, 删除残留文件重新下载...")
                os.remove(fileName)
                return False

        download_file_size = get_file_size(link)  # 下载完成后检查文件大小
        local_file_size = os.path.getsize(fileName)

        if download_file_size != local_file_size:
            print(f"download_file_size: {download_file_size}     local_file_size: {local_file_size}")
            print("下载后的文件大小与实际的不相等")

            print("删除不完整文件")
            os.remove(fileName)
            # 重试重新下载  (暂时没添加)
            return False

    print("正在解压文件....")
    file_name = softwarePath.split('\\')[-1]
    if os.path.exists(f"{root_path}data\\{file_name}"):
        shutil.rmtree(f"{root_path}data\\{file_name}")
        sleep(2)
    try:
        patoolib.extract_archive(fileName, outdir=f"{root_path}data\\{file_name}")
        if len(get_directories(softwarePath)) == 1:
            softwarePath = softwarePath + "\\" + get_directories(softwarePath)[0]
    except Exception as e:
        print("解压失败")
        print(e)
        return False

    # 复制出默认items.ini
    if os.path.exists(f'{softwarePath}\\items.ini'):
        print("正在修改items")
        if not os.path.exists(f"{root_path}data\\items"):
            # 创建文件夹
            os.mkdir(f"{root_path}data\\items")
        file_name = softwarePath.split("\\")[-1]
        shutil.copy(f'{softwarePath}\\items.ini', f"{root_path}data\\items\\{file_name}_items.ini")

        modify_type = modify_items(softwarePath, jointly_model=jointly_model, sale_model=sale_model)
        if not modify_type:
            print("items修改失败")

    if os.path.exists(softwarePath + '\\Checksum.ini'):
        print("正在删除Checksum.ini文件....")
        os.remove(softwarePath + '\\Checksum.ini')
    return softwarePath


def modify_items(file_path, jointly_model, sale_model):
    """修改items"""
    update_path = None
    update_model = get_directories(rf"{root_path}data\update_items")
    for model in update_model:
        if model in file_path:
            update_path = rf"{root_path}data\update_items\{model}\items.ini"
            print(f"需要修改items文件路径：{update_path}")

    if not update_path:
        print("备份items 不存在")
        return False

    print("items 加密机型执行复制替换")
    file_path = file_path + r'\items.ini'
    shutil.copy2(file_path, update_path)

    file_items_path = file_path + '\\items.ini'
    if not os.path.exists(file_items_path):
        print(f"不存在此文件，修改itmes失败: {file_items_path}")
        return False

    print("正在修改items文件.....")
    with open(file_items_path, 'r') as f:
        line_list = f.readlines()
        if "ro.seewo.usb.adb" not in str(line_list):
            print("修改 默认 开启 adb")
            line_list.insert(line_list.index("items.end\n"), "persist.sys.seewo.develop.adb                   1\n")
            line_list.insert(line_list.index("items.end\n"), "ro.seewo.usb.adb                    1\n")
            line_list.insert(line_list.index("items.end\n"), "service.adb.tcp.port                    5555\n")
            line_list.insert(line_list.index("items.end\n"),
                             "persist.sys.seewo.monkey         true\n")  # monkey日志缓存增加
            line_list.insert(line_list.index("items.end\n"),
                             "persist.sys.mtklog.need.alert         1\n")  # 解决mtklog 弹窗警告问题

        if jointly_model or sale_model:
            file_name = file_path.split(r'\\')[-1]
            shutil.copy(f"{root_path}data\\items\\{file_name}_items.ini", file_items_path)  # 备份

        if jointly_model and len(str(jointly_model).strip()) >= 1:
            print("修改联名型号")
            for index, red in enumerate(line_list):
                if "ro.seewo.tags" in red:
                    line_list[index] = f'ro.seewo.tags {jointly_model}\n'
        if sale_model and len(str(sale_model).strip()) >= 1:
            print("修改销售型号")
            for index, red in enumerate(line_list):
                if "ro.seewo.sales.model" in red:
                    line_list[index] = f'ro.seewo.sales.model {sale_model}\n'

    with open(file_path + '\\items.ini', 'w', newline='\n') as f:
        f.writelines(line_list)



# coding: utf-8
# Project：xbsSysStabilityAT
# File：mtk.py
# Author：张炼武
# Date ：2025/4/27 11:48
# IDE：PyCharm
import os
import queue
import threading
from time import sleep
from xbsTestTools.system_tools.buring.download_tool import get_download_file, cmd_to_file, query_update_folder, getMtkLog_keywords


def mtk_flash(dev, softwarePath, burn_type):
    """
    烧录方法
    dev 设备device
    message 烧录消息
    _type 烧录状态
    softwarePath 烧录软件地址
    """
    root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__))) + os.sep
    flashTool = rf"{root_path}utils\SP_Flash_Tool\flash_tool.exe"
    daFile = rf"{root_path}utils\SP_Flash_Tool\MTK_ALLInOne_DA.bin"
    # 软件Path
    mirror_file = get_download_file("MTK", softwarePath)
    scatterFile = os.path.join(softwarePath, mirror_file)
    flashCmd = flashTool + ' -c firmware-upgrade -b -d {} -s {}'.format(daFile, scatterFile)
    print(f"{dev} 执行MTK烧录命令: {flashCmd}")
    # 执行cmd命令并输出txt
    cmdType, value = cmd_to_file(flashCmd, "flashRead.txt", 400)
    if not cmdType:
        print(f"{dev} 烧写版本超时")
        burn_type.put(False)
        return

    print("解析MTK烧录Log")
    # 获取MTK最新日志
    newTimeFolder = query_update_folder(r'C:\ProgramData\SP_FT_Logs')

    if newTimeFolder is None:
        print(f"{dev} 未能获取到MTK烧录器中最新的日志文件")
    else:
        # 检查是否烧录成功
        if not getMtkLog_keywords(newTimeFolder):
            print(f"{dev} 烧写失败，没找到烧录成功关键词")
            burn_type.put(False)
            return

    print(f"{dev} 烧写成功%s")
    burn_type.put(True)


def mtk_burn_run(dev, softwarePath):
    print(f"{dev} 开始烧录")
    burn_type = queue.Queue()
    # 烧录启动
    t = threading.Thread(target=mtk_flash, args=(dev, burn_type, softwarePath))
    t.start()
    sleep(10)
    print(f"{dev} 进入下载模式")
    os.system(f'adb -s {dev} reboot')
    print(f"{dev} 版本烧录中")

    t.join()
    print(f"{dev} 烧录进程结束")
    sleep(20)
    return burn_type.get()



# coding: utf-8
# Project：xbsTestTools
# File：test_ui_tools.py
# Author：张炼武
# Date ：2025/6/9 11:10
# IDE：PyCharm
class TestUiTools:
    def __init__(self, dev):
        self.dev = dev

    def skip_guide(self):
        """跳过引导"""
        ...

    def login_account(self):
        """登录账号"""
        ...


# coding: utf-8
# Project：xbsTestTools
# File：__init__.py.py
# Author：张炼武
# Date ：2025/5/15 16:41
# IDE：PyCharm

from .adb_tools import AdbTools
from .format_execution import FormatExecution
from .method_executor import MethodExecutor
from .operate_controller import XbsUiOperate, XbsUiTest
from .try_error import try_catch
from .log import log
from .system_tools.test_tools import TestTools

__all__ = ['AdbTools', 'FormatExecution', 'MethodExecutor', 'XbsUiOperate', 'XbsUiTest', 'try_catch', 'log',
           'TestTools']
